//! Cursor implementation
use crate::{AccountId, BaseTx, Filter, RevId, Revision, Tag, TxId, Type};

#[derive(Debug, Clone, PartialEq, Eq, Hash)]
/// The primary filter is used to filter the transactions before applying the other filters. Think
/// of it like which key is used to filter transactions quickly before applying the other filters.
pub enum PrimaryFilter {
    /// By transaction ID
    TxId(TxId),
    /// By revision ID
    Revision(RevId),
    /// By accounts
    Account(AccountId),
    /// By transaction type
    Type(Type),
    /// By tags
    Tags(Tag),
    /// By transaction status
    Stream,
}

#[derive(Debug, PartialEq, Eq, Hash, Clone)]
/// The cursor
pub struct Cursor {
    /// The primary filter
    pub primary_filter: Vec<PrimaryFilter>,
    /// The secondary filter
    pub filter: Filter,
}

impl From<Filter> for Cursor {
    fn from(mut filter: Filter) -> Self {
        let primary_filter = if !filter.revisions.is_empty() {
            std::mem::take(&mut filter.revisions)
                .into_iter()
                .map(|r| PrimaryFilter::Revision(r))
                .collect()
        } else if !filter.ids.is_empty() {
            std::mem::take(&mut filter.ids)
                .into_iter()
                .map(|id| PrimaryFilter::TxId(id))
                .collect()
        } else if !filter.accounts.is_empty() {
            std::mem::take(&mut filter.accounts)
                .into_iter()
                .map(|account| PrimaryFilter::Account(account))
                .collect()
        } else if !filter.typ.is_empty() {
            std::mem::take(&mut filter.typ)
                .into_iter()
                .map(|typ| PrimaryFilter::Type(typ))
                .collect()
        } else if !filter.tags.is_empty() {
            std::mem::take(&mut filter.tags)
                .into_iter()
                .map(|tag| PrimaryFilter::Tags(tag))
                .collect()
        } else {
            vec![PrimaryFilter::Stream]
        };

        Self {
            primary_filter,
            filter: filter.prepare(),
        }
    }
}

impl Cursor {
    /// Check if the base transaction and the revision matches the current cursor filter
    pub fn matches(&self, base_tx: &BaseTx, revision: &Revision) -> bool {
        self.filter.matches(base_tx, revision)
    }
}
